package version9_2.Interface;

import version9_2.environment.Environment;
import version9_2.platform.Agent;

/**
 * convert signals from the robot into interactions
 * @author simon
 */
public class Perception {

	private Agent agent;

	private int size=3;       						// nb of status
	private boolean[] status;
	
	public Perception(Agent a){
		agent=a;
		
		// "physical" robot input
		status=new boolean[size];
		for (int i=0;i<size;i++){
			status[i]=false;
		}
	}

	
	public void update(){
		
		//%%%%%%%%%%%%%%%% specific to render function %%%%%%%%%%%%%%%%%%%
		agent.probe.rendu();
		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		// get the input vector
		for (int i=0;i<size;i++) status[i]=false;
		if (agent.body.sensors[0]==1) status[0]=true;
		if (agent.body.sensors[0]==2) status[1]=true;
		if (agent.body.sensors[5]==1) status[2]=true;
	}
	
	
	// interpretation of enacted interaction according to action and inputs
	public PrimitiveInteraction recognize(PrimitiveInteraction intended){
		
		if (intended.getAction().getName().equals(">")){
			if (status[2]) 		return agent.interactionList.getInteraction(2);
			else if (status[1]) return agent.interactionList.getInteraction(1);
			else 				return agent.interactionList.getInteraction(0);
		}
		else{
			return intended;
		}
	}
	
	public float[][] recognizeEnactedEnsemble(PrimitiveInteraction enacted){

		float[][] enactedEnsemble=new float[9][InteractionList.length];
		
		// initialize enacted ensemble
		for (int i=0;i<InteractionList.length;i++){
			for (int m=0;m<9;m++){
				enactedEnsemble[m][i]=0;
			}
		}
		
		// set primary interaction
		enactedEnsemble[0][enacted.getIndex()+InteractionList.nbDF]=1;
		
		// set secondary interactions
		for (int i=0;i<agent.visual.color.length;i++){
			if (agent.visual.opticFlow[i]!=0 && agent.visual.opticFlow[i]<49){
				int col=0;
				if (agent.visual.color[i].equals(Environment.WALL1)) col=1;
				else if (agent.visual.color[i].equals(Environment.FISH1)) col=2;
				else if (agent.visual.color[i].equals(Environment.ALGA1)) col=3;
				
				int mov=4;
				if (agent.visual.flow[0][i]!=0){
					double rz=agent.visual.flow[1][i];
					if (rz>=  Math.PI/8 && rz<3*Math.PI/8) mov=0;
					if (rz>=3*Math.PI/8 && rz<5*Math.PI/8) mov=1;
					if (rz>=5*Math.PI/8 && rz<7*Math.PI/8) mov=2;
					if (rz>=7*Math.PI/8 && rz<9*Math.PI/8) mov=5;
					if (rz>=9*Math.PI/8 && rz<11*Math.PI/8) mov=8;
					if (rz>=11*Math.PI/8 && rz<13*Math.PI/8)mov=7;
					if (rz>=13*Math.PI/8 && rz<15*Math.PI/8)mov=6;
					if (rz>=15*Math.PI/8 || rz<  Math.PI/6) mov=3;
				}
				
				if (col!=0){

					float theta=(float) Math.toRadians((i*5)-90);
					
					float x= (float) ( agent.visual.opticFlow[i]*Math.sin(theta))+InteractionList.size1/2;
					float y= (float) ( agent.visual.opticFlow[i]*Math.cos(theta));

					
					
					for (int i2=-2;i2<=2;i2++){
						for (int j2=-2;j2<=2;j2++){
							if ((int)x+i2>=0 && (int)x+i2<InteractionList.size1
							 && (int)y+j2>=0 && (int)y+j2<InteractionList.size2){
								
								float d=(float) Math.sqrt(i2*i2+j2*j2);
								if (d<1){
									for (int k=0;k<3;k++){
										if (k==col-1)  enactedEnsemble[mov][((int)x+i2)*InteractionList.size2*3 + ((int)y+j2)*3 + k]=1;
									}
								}
								else{
									for (int k=0;k<3;k++){
										if (k==col-1){

											enactedEnsemble[mov][((int)x+i2)*InteractionList.size2*3 + ((int)y+j2)*3 + k]=
												Math.max(enactedEnsemble[mov][((int)x+i2)*InteractionList.size2*3 + ((int)y+j2)*3 + k], 1/(d*2));
										}
									}
								}
							}
						}
					}
				}
			}
		}
		
		// bias input
		enactedEnsemble[0][InteractionList.length-1]=1;
		
		return enactedEnsemble;
	}
	
}
