package version9_2.display;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import version9_2.Interface.InteractionList;
import version9_2.platform.Agent;
import version9_2.spaceMemory.BlurPattern;
import version9_2.spaceMemory.Signature;

/**
 * Display space memory data
 * @author simon gay
 */

/* - inherited from EnvPanel :
 *   Agent agent      : pointer to the agent
 */
public class SignaturePanel extends EnvPanel implements MouseListener{

	private static final long serialVersionUID = 1L;
	
	private int clic_x=0;
	private int clic_y=0;
	private int selected_interaction=0;
	
	public SignaturePanel(Agent a){
		super(a);
		addMouseListener(this);
	}
	
	public void paintComponent(Graphics g){
		g.setColor(Color.white);
		
		g.fillRect(0, 0, 1350, 1000);
		
		
		// list of interactions
		for (int i=0;i<agent.spaceMemory.nbInteraction();i++){
			if (selected_interaction==i) g.setColor(Color.red);
			else g.setColor(Color.black);
			g.fillOval(20+((int)(i/15)*250),600+15*(i%15),5,5);
			
			g.setColor(Color.black);
			g.drawString(agent.spaceMemory.getInteraction(i).getName(1), 30+((int)(i/15)*250), 610+15*(i%15));
		}

		
		/////////////////////////////////////////////////////////////////////////////////
		// draw signatures
		if (selected_interaction!=-1){
			int ident=selected_interaction;
			Signature s=agent.spaceMemory.getInteraction(ident).pattern;
			BlurPattern blur=agent.spaceMemory.getInteraction(ident).blur;
			float max1=s.maxPattern1;
			float max2=s.maxPattern2;

			if (max1!=0){
				
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2;j++){
						for (int m=0;m<9;m++){
							g.setColor(new Color(Math.min(1, Math.max(0, s.pattern[m][i*InteractionList.size2*3+j*3+2]/max1+1)/2),
												 Math.min(1, Math.max(0, s.pattern[m][i*InteractionList.size2*3+j*3  ]/max1+1)/2),
												 Math.min(1, Math.max(0, s.pattern[m][i*InteractionList.size2*3+j*3+1]/max1+1)/2)));
							
							g.fillRect(10+i*2 +210*( (int)(m/3) ), 110-j*2 + 110*(m%3), 2, 2);
						}
					}
				}
				
				
				// blur pattern
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int m=0;m<9;m++){
							g.setColor(new Color(Math.max(0, Math.min(1, (5*blur.pattern[i][j][m][2]+1)/2)),
												 Math.max(0, Math.min(1, (5*blur.pattern[i][j][m][0]+1)/2)),
												 Math.max(0, Math.min(1, (5*blur.pattern[i][j][m][1]+1)/2))));
							
							g.fillRect(640+i*2 +210*( (int)(m/3) ),210-j*2 + 210*(m%3), 2, 2);
						}
					}
				}
			}
			
			if (max2!=0){
				
				// display values of weights related to primary interactions and bias
				g.setColor(Color.black);
				for (int i=0;i<8;i++){
					g.drawString(""+s.pattern[0][InteractionList.nbDF+i], 50, 480+i*10);
				}
				
				for (int i=0;i<8;i++){
					g.setColor(new Color(Math.max(0, Math.min(1,(1-((s.pattern[0][InteractionList.size1*InteractionList.size2*3+i]/max2/2)+0.5f)))),
							             Math.max(0, Math.min(1,(    s.pattern[0][InteractionList.size1*InteractionList.size2*3+i]/max2/2)+0.5f)),
							             0));
					g.fillRect(20+i*15, 350, 10, 10);
				}
			}	
		}
	}
	

	///////////////////////////////////////////////////////////////////////////
	public void mouseClicked(MouseEvent e) {

		clic_x=e.getX();
		clic_y=e.getY();
		
		float dmin=10;
		float d=10;
		
		selected_interaction=-1;
		
		int nbCol=(int)(agent.spaceMemory.nbInteraction()/15);
		int nbLast=agent.spaceMemory.nbInteraction()%15;
		
		for (int i=0;i<nbCol;i++){
			for (int j=0;j<15;j++){
				d= (20+250*i-clic_x)*(20+250*i-clic_x) + (600+j*15-clic_y)*(600+j*15-clic_y);
				d=(float) Math.sqrt(d);
				
				if (d<dmin){
					dmin=d;
					selected_interaction=i*15+j;
				}
			}
		}
		
		for (int j=0;j<nbLast;j++){
			d= (20+250*nbCol-clic_x)*(20+250*nbCol-clic_x) + (600+j*15-clic_y)*(600+j*15-clic_y);
			d=(float) Math.sqrt(d);
			
			if (d<dmin){
				dmin=d;
				selected_interaction=nbCol*15+j;
			}
		}

		if (clic_x>=(305) && clic_x<=(315) && clic_y>=220 && clic_y<=230) 
			agent.decision.setLearning();
	}

	@Override
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	
}
