package version9_2.spaceMemory;

import version9_2.Interface.InteractionList;
import version9_2.Interface.PrimitiveInteraction;
import version9_2.environment.Prey;
import version9_2.platform.Agent;

/**
 * Algorithmic simplification for object instance detection
 * @author simon
 */
public class BlurPattern {

	public float[][][][] pattern;
	private int counter=0;
	
	private static int cycle=100;
	
	public BlurPattern(){
		pattern=new float[InteractionList.size1][InteractionList.size2*2][9][3];
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2*2;j++){
				for (int m=0;m<9;m++){
					for (int k=0;k<3;k++){
						pattern[i][j][m][k]=0;
					}
				}
			}
		}
	}
	
	
	public void generateBlur(Signature s, Agent a){
		if (counter==0) blur(s,a);
		counter++;
		if (counter==cycle) counter=0;
	}
	
	public void blur(Signature s, Agent agent){

		// initialize blur pattern
		for (int m=0;m<9;m++){
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2;j++){
					for (int k=0;k<3;k++){
						if (Math.abs(s.pattern[m][i*InteractionList.size2*3 + j*3 + k])>0.1 && Math.abs(s.pattern[m][i*InteractionList.size2*3 + j*3 + k])>s.maxPattern1*0.5)
							pattern[i][j+InteractionList.size2][m][k]=s.pattern[m][i*InteractionList.size2*3 + j*3 + k];
					}
				}
			}
		}
		
		// generate the blur pattern
		
		float[][][][] buffer=new float[InteractionList.size1][InteractionList.size2*2][9][3];
		
		for (int n=0;n<10;n++){
			
			// copy current blur pattern
			for (int m=0;m<9;m++){
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int k=0;k<3;k++){
							buffer[i][j][m][k]=pattern[i][j][m][k];
						}
					}
				}
			}
			
			for (int inter=0;inter<7;inter++){
				for (int m=0;m<9;m++){
					for (int i=0;i<InteractionList.size1;i++){
						for (int j=0;j<InteractionList.size2*2;j++){
							
							int x=(int) (agent.interactionList.getInteraction(inter).vectorFlow2[i][j][0]);
							int y=(int) (agent.interactionList.getInteraction(inter).vectorFlow2[i][j][1]);
		
							int mx=(int)(m/3)-1;
							int my=m%3-1;
							
							if (mx==0 || my==0){
								x+=mx*PrimitiveInteraction.forward*Prey.speed;
								y+=my*PrimitiveInteraction.forward*Prey.speed;
							}
							else{
								x+=mx*PrimitiveInteraction.forward*Prey.speed/Math.sqrt(2);
								y+=my*PrimitiveInteraction.forward*Prey.speed/Math.sqrt(2);
							}
							
							if (x>=0 && y>=0 && x<InteractionList.size1 && y<InteractionList.size2*2){
								for (int k=0;k<3;k++){
									if (Math.abs(0.5*buffer[x][y][permutation(inter,m)][k])>Math.abs(pattern[i][j][m][k]))
										pattern[i][j][m][k]=0.5f*buffer[x][y][permutation(inter,m)][k];
								}
							}
						}
					}
				}
			}
		}
	}
	
	private int permutation(int inter, int mov){
		if (inter<3) return mov;
		else{
			if (inter==3){ //case turn right 90°
				switch (mov){
				case 0: return 2;
				case 1: return 5;
				case 2: return 8;
				case 3: return 1;
				case 4: return 4;
				case 5: return 7;
				case 6: return 0;
				case 7: return 3;
				case 8: return 6;
				default: return 4;
				}
			}
			else if (inter==4){ // case turn left 90°
				switch (mov){
				case 0: return 6;
				case 1: return 3;
				case 2: return 0;
				case 3: return 7;
				case 4: return 4;
				case 5: return 1;
				case 6: return 8;
				case 7: return 5;
				case 8: return 2;
				default: return 4;
				}
			}
			else if (inter==5){ // case turn right 45°
				switch (mov){
				case 0: return 1;
				case 1: return 0;
				case 2: return 5;
				case 3: return 0;
				case 4: return 4;
				case 5: return 8;
				case 6: return 3;
				case 7: return 6;
				case 8: return 7;
				default: return 4;
				}
			}
			else if (inter==6){ // case turn left 45°
				switch (mov){
				case 0: return 3;
				case 1: return 2;
				case 2: return 1;
				case 3: return 6;
				case 4: return 4;
				case 5: return 2;
				case 6: return 7;
				case 7: return 8;
				case 8: return 5;
				default: return 4;
				}
			}
			else return mov;
		}
	}
	
}
