package version9_2.spaceMemory;

import version9_2.Interface.InteractionList;

/**
 * signature of an interaction
 * @author simon gay
 */
public class Signature {

	public static float learning_rate=0.01f;
	
	public float[][] pattern;
	
	public float maxPattern;				// maximum weight value (in absolute value), used for display color normalization
	public float maxPattern1;				// maximum weight value of visual interactions
	public float maxPattern2;				// maximum weight value of primary interactions
	
	public float lastDelta1;
	public float lastDelta2;
	
	private int counter;
	
	private float prediction=0;
	
	public Signature(){
		pattern=new float[10][InteractionList.length];
		for (int i=0;i<InteractionList.length;i++){
			for (int m=0;m<10;m++){
				pattern[m][i]=0;
			}
		}
		maxPattern=0;
		maxPattern1=0;
		maxPattern2=0;
		
		counter=0;
	}
	
	/**
	 * predict the result of an interaction in a given context
	 * @param img : environmental context
	 * @param write : if true, store the prediction
	 * @return the certitude of success (1->absolute certitude of success, -1-> absolute certitude of failure
	 */
	public float prediction(float[][] img, boolean write){
		float result=0;
		for (int i=0;i<InteractionList.length;i++){
			for (int m=0;m<9;m++){
				result+=pattern[m][i]*img[m][i];
			}
		}
		result= (float) ( 1 / (1+exp(-result)))*2-1;
		
		if (write) prediction=result;
		
		return result;
	}
	
	/**
	 * integrate the result of enaction cycle
	 * @param img : environmental context E_{t-1} 
	 * @param output result of enaction (success->1, failure->-1)
	 */
	public void learn(float[][] img, float output){
		maxPattern1=0;
		maxPattern2=0;
		float delta=output-prediction;

		if (output>0) lastDelta1=delta;
		else		  lastDelta2=delta;
		
		if (delta>0.5) counter=0;
		if (counter<100) counter++;
		
		for (int i=0;i<InteractionList.length;i++){
			for (int m=0;m<9;m++){
				pattern[m][i]+= learning_rate * delta *Math.abs(delta) * img[m][i];
				
				if (Math.abs(pattern[m][i])>maxPattern) maxPattern=Math.abs(pattern[m][i]);
				if (i< InteractionList.nbDF && Math.abs(pattern[m][i])>maxPattern1) maxPattern1=Math.abs(pattern[m][i]);
				if (i>=InteractionList.nbDF && Math.abs(pattern[m][i])>maxPattern2) maxPattern2=Math.abs(pattern[m][i]);
			}
		}
		
		// limit the bias value, as the range of weights is limited by the 'short' type
		if (pattern[0][InteractionList.length-1]> 6){
			pattern[0][InteractionList.length-1]= 6;
			for (int i=0;i<InteractionList.length-1;i++){
				for (int j=0;j<9;j++){
					if (img[j][i]>0) pattern[j][i]=pattern[j][i]*0.9f;
				}
			}
		}
		if (pattern[0][InteractionList.length-1]<-6){
			pattern[0][InteractionList.length-1]=-6;
			for (int i=0;i<InteractionList.length-1;i++){
				for (int j=0;j<9;j++){
					if (img[j][i]>0) pattern[j][i]=pattern[j][i]*0.9f;
				}
			}
		}
		
	}
	
	public float getPrediction(){
		return prediction;
	}
	
	public boolean isReliable(){
		return counter>=100;
	}
	
	// fast exp function approximation
	public static double exp(double val) {
	    final long tmp = (long) (1512775 * val + (1072693248 - 60801));
	    return Double.longBitsToDouble(tmp << 32);
	}
	
}
