package version9_2.spaceMemory;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;

import version9_2.environment.Prey;
import version9_2.platform.Agent;
import version9_2.platform.Observer;
import version9_2.Interface.InteractionList;
import version9_2.Interface.PrimitiveInteraction;


/**
 * Main class of the space memory mechanism
 * @author simon gay
 */
public class SpaceMemory {

	private Agent agent;

	public float[][][] enactedEnsemblesM;			// sequence of the last 'timesize' enacted ensembles including movements
	
	public int[][][] counters;						// nb of test for each input and for each interaction

	public float[][][][] spaceMap;					// map used by the memory mechanism
	
	public ArrayList<Float[][][][]> predictionMap;	// map predicted for each primitive interaction
	
	private ArrayList<Float> additional;			// utility values added to satisfaction values
	
	private ArrayList<Composite> interactionList;	// list of known composite interactions
	
	private PrimitiveInteraction[] timeline;		// timeline of the last enacted primary interactions
	
	public static int timeSize=1;
	
	public static float memoryCoef = 0.5f;    	// influence coefficient of the space memory
	//public static float objectCoef = 0.4f;     	// influence of objects according to their distances

	private float motion=PrimitiveInteraction.forward*Prey.speed;
	
	public static boolean load=true;			// if true, load an agent file
	
	
	public SpaceMemory(Agent a){
		agent=a;
		
		enactedEnsemblesM=new float[timeSize+1][9][InteractionList.length];		// no movement + 8 movements

		spaceMap=new float[InteractionList.size1][InteractionList.size2*2][9][3];
		
		predictionMap=new ArrayList<Float[][][][]>();
		additional=new ArrayList<Float>();
		
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2*2;j++){
				for (int m=0;m<9;m++){
					for (int k=0;k<3;k++){
						spaceMap[i][j][m][k]=0;
					}
				}
			}
		}
		
		counters=new int[7][9][InteractionList.length];
		for (int i=0;i<7;i++){
			for (int j=0;j<9;j++){
				for (int k=0;k<InteractionList.length;k++){
					counters[i][j][k]=25;
				}
			}
		}
		
		interactionList=new ArrayList<Composite>();

		timeline=new PrimitiveInteraction[timeSize];
		for (int i=0;i<timeSize;i++){
			timeline[i]=null;
		}

		if (load) loadFile();
		else load();
	}
	
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////
	// main space memory update function
	public void updateMemory(PrimitiveInteraction inter, float[][] enacted){
		
		System.out.println("SpaceMemory observed primitive "+inter.getName());
		
		// update timeline
		for (int i=timeSize-1;i>0;i--){
			timeline[i]=timeline[i-1];
		}
		timeline[0]=inter;
		
		// update counters
		for (int j=0;j<9;j++){
			for (int k=0;k<InteractionList.length;k++){
				if (counters[inter.getIndex()][j][k]>0 && enactedEnsemblesM[0][j][k]>0){
					counters[inter.getIndex()][j][k]--;
				}
			}
		}
		
		// display timeline
		System.out.print("time line : ");
		for (int i=timeSize-1;i>=0;i--){
			if (timeline[i]!=null) System.out.print(timeline[i].getName()+", ");
			else System.out.print("null , ");
		}
		System.out.println();
		
		////////////////////////////////////////////////
		// save previous memory and reset current memory
		float[][][][] oldMap=new float[InteractionList.size1][InteractionList.size2*2][9][3];
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2*2;j++){
				for (int m=0;m<9;m++){
					for (int k=0;k<3;k++){
						oldMap[i][j][m][k]=spaceMap[i][j][m][k];
						spaceMap[i][j][m][k]=0;
					}
				}
			}
		}
		
		// update stored elements
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2*2;j++){
				int x=Math.round(inter.vectorFlow[i][j][0]);
				int y=Math.round(inter.vectorFlow[i][j][1]);
					
				// update map for each movement
				// forward-left
				if (x+(int)(motion*Math.sqrt(2))>=0 && x+(int)(motion*Math.sqrt(2))<InteractionList.size1 
				 && y-(int)(motion*Math.sqrt(2))>=0 && y-(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x+(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][0][k])>Math.abs(spaceMap[i][j][0][k]) 
						 && Math.abs(oldMap[x+(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][0][k])>0.5){
							spaceMap[i][j][0][k]=0.99f*oldMap[x+(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][0][k];
						}
					}
				}
				
				// left
				if (x+motion>=0 && x+motion<InteractionList.size1 && y>=0 && y<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x+(int)motion][y][1][k])>Math.abs(spaceMap[i][j][1][k]) && Math.abs(oldMap[x+(int)motion][y][1][k])>0.5){
							spaceMap[i][j][1][k]=0.99f*oldMap[x+(int)motion][y][1][k];
						}
					}
				}
				
				// backward-left
				if (x+(int)(motion*Math.sqrt(2))>=0 && x+(int)(motion*Math.sqrt(2))<InteractionList.size1 
				 && y+(int)(motion*Math.sqrt(2))>=0 && y+(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x+(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][2][k])>Math.abs(spaceMap[i][j][2][k]) 
						 && Math.abs(oldMap[x+(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][2][k])>0.5){
							spaceMap[i][j][2][k]=0.99f*oldMap[x+(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][2][k];
						}
					}
				}
				
				// forward
				if (x>=0 && x<InteractionList.size1 && y-(int)motion>=0 && y-(int)motion<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x][y-(int)motion][3][k])>Math.abs(spaceMap[i][j][3][k]) && Math.abs(oldMap[x][y-(int)motion][3][k])>0.5){
							spaceMap[i][j][3][k]=0.99f*oldMap[x][y-(int)motion][3][k];
						}
					}
				}
				
				// no movement
				if (x>=0 && x<InteractionList.size1 && y>=0 && y<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x][y][4][k])>Math.abs(spaceMap[i][j][4][k]) && Math.abs(oldMap[x][y][4][k])>0.5){
							spaceMap[i][j][4][k]=0.99f*oldMap[x][y][4][k];
						}
					}
				}
				
				// backward
				if (x>=0 && x<InteractionList.size1 && y+motion>=0 && y+motion<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x][y+(int)motion][5][k])>Math.abs(spaceMap[i][j][5][k]) && Math.abs(oldMap[x][y+(int)motion][5][k])>0.5){
							spaceMap[i][j][5][k]=0.99f*oldMap[x][y+(int)motion][5][k];
						}
					}
				}
				
				// forward-right
				if (x-(int)(motion*Math.sqrt(2))>=0 && x-(int)(motion*Math.sqrt(2))<InteractionList.size1 
				 && y-(int)(motion*Math.sqrt(2))>=0 && y-(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x-(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][6][k])>Math.abs(spaceMap[i][j][6][k]) 
						 && Math.abs(oldMap[x-(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][6][k])>0.5){
							spaceMap[i][j][6][k]=0.99f*oldMap[x-(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][6][k];
						}
					}
				}
				
				// right
				if (x-motion>=0 && x+motion<InteractionList.size1 && y>=0 && y<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x-(int)motion][y][7][k])>Math.abs(spaceMap[i][j][7][k]) && Math.abs(oldMap[x-(int)motion][y][7][k])>0.5){
							spaceMap[i][j][7][k]=0.99f*oldMap[x-(int)motion][y][7][k];
						}
					}
				}
				
				// backward-right
				if (x-(int)(motion*Math.sqrt(2))>=0 && x-(int)(motion*Math.sqrt(2))<InteractionList.size1 
				 && y+(int)(motion*Math.sqrt(2))>=0 && y+(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
					for (int k=0;k<3;k++){
						if (Math.abs(oldMap[x-(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][8][k])>Math.abs(spaceMap[i][j][8][k]) 
						 && Math.abs(oldMap[x-(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][8][k])>0.5){
							spaceMap[i][j][8][k]=0.99f*oldMap[x-(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][8][k];
						}
					}
				}
				
			}
		}
		
		//////////////////////////////////////////////////////////
		// permutation of movement memories when the agent rotates
		float[][][] tempMap=new float[InteractionList.size1][InteractionList.size2*2][3];
		
		// turn left 90°
		if (inter.getIndex()==4){
			System.out.println(" ++++ turn left 90");
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						tempMap[i][j][c]=spaceMap[i][j][3][c];			// save forward map
					}
				}
			}
			permuteMaps(1,3);
			permuteMaps(5,1);
			permuteMaps(7,5);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						spaceMap[i][j][7][c]=tempMap[i][j][c];			// save forward map
					}
				}
			}
			
			
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						tempMap[i][j][c]=spaceMap[i][j][0][c];			// save forward map
					}
				}
			}
			permuteMaps(2,0);
			permuteMaps(8,2);
			permuteMaps(6,8);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						spaceMap[i][j][6][c]=tempMap[i][j][c];			// save forward map
					}
				}
			}
		}
		
		
		// turn right 90°
		if (inter.getIndex()==3){
			System.out.println(" ++++ turn right 90");
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						tempMap[i][j][c]=spaceMap[i][j][3][c];			// save forward map
					}
				}
			}
			permuteMaps(7,3);
			permuteMaps(5,7);
			permuteMaps(1,5);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						spaceMap[i][j][1][c]=tempMap[i][j][c];			// save forward map
					}
				}
			}
			
			
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						tempMap[i][j][c]=spaceMap[i][j][0][c];			// save forward map
					}
				}
			}
			permuteMaps(6,0);
			permuteMaps(8,6);
			permuteMaps(2,8);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						spaceMap[i][j][2][c]=tempMap[i][j][c];			// save forward map
					}
				}
			}
		}
		
		
		// turn left 45°
		if (inter.getIndex()==6){
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						tempMap[i][j][c]=spaceMap[i][j][3][c];			// save forward map
					}
				}
			}
			permuteMaps(0,3);
			permuteMaps(1,0);
			permuteMaps(2,1);
			permuteMaps(5,2);
			permuteMaps(8,5);
			permuteMaps(7,8);
			permuteMaps(6,7);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						spaceMap[i][j][6][c]=tempMap[i][j][c];			// save forward map
					}
				}
			}
		}
		
		// turn right 45°
		if (inter.getIndex()==5){
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						tempMap[i][j][c]=spaceMap[i][j][3][c];			// save forward map
					}
				}
			}
			permuteMaps(6,3);
			permuteMaps(7,6);
			permuteMaps(8,7);
			permuteMaps(5,8);
			permuteMaps(2,5);
			permuteMaps(1,2);
			permuteMaps(0,1);
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int c=0;c<3;c++){
						spaceMap[i][j][0][c]=tempMap[i][j][c];			// save forward map
					}
				}
			}
		}
		
		
		///////////////////////////////////////////////////////////////////////////////////////////
		// compute enacted ensemble
		for (int i=0;i<InteractionList.length;i++){
			for (int m=0;m<9;m++){
				enactedEnsemblesM[1][m][i]=enactedEnsemblesM[0][m][i];
				enactedEnsemblesM[0][m][i]=enacted[m][i];
			}
		}

		// add new elements to the space memory
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2;j++){
				for (int m=0;m<9;m++){
					for (int c=0;c<3;c++){
						if (Math.abs(enactedEnsemblesM[0][m][i*InteractionList.size2*3 + j*3 + c])>Math.abs(spaceMap[i][j][m][c])){
							spaceMap[i][j+InteractionList.size2][m][c]=enactedEnsemblesM[0][m][i*InteractionList.size2*3 + j*3 + c];
						}
					}
				}
			}
		}
		
		
		// read timeline and learn new sequences
		int t=0;
		while (t<timeSize && timeline[t]!=null){
			PrimitiveInteraction[] temp=new PrimitiveInteraction[t+1];
			for (int i=0;i<=t;i++) temp[i]=timeline[i];
			learn(temp);
			t++;
		}
		
		for (int i=0;i<interactionList.size();i++){
			
			if (interactionList.get(i).checkSequence(timeline)!=0){
				interactionList.get(i).pattern.learn(
						enactedEnsemblesM[interactionList.get(i).size()],
						interactionList.get(i).checkSequence(timeline)
						);
				interactionList.get(i).blur.generateBlur(getInteraction(i).pattern, agent);
			}
			interactionList.get(i).setPrediction(enactedEnsemblesM[0]);
		}
		
		// generate predicted memories
		prediction();
		
		// compute global proximity values for each interaction
		for (int i=0;i<interactionList.size();i++){
			interactionList.get(i).setPrediction1(agent.interactionList.nbInteraction(),spaceMap);
			interactionList.get(i).setPrediction2(predictionMap);
		}
		
		// compute the additional satisfaction value
		for (int i=0;i<agent.interactionList.nbInteraction();i++){
			additional.set(i,0f);
			if (interactionList.get(i).getPrediction()>=0){
				for (int j=0;j<7;j++){
					if (i!=j && interactionList.get(j).predictions1.get(i)!=0){
						additional.set(i,additional.get(i)
								        + interactionList.get(j).getValence()
						                 *(interactionList.get(j).predictions2.get(i)-interactionList.get(j).predictions1.get(i))
						               );
					}
				}
			}
		}
	}
	
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	////////////////////////////////
	// add new Composite interaction
	private int learn(PrimitiveInteraction[] seq){

		// sequences and patterns
		int found=-1;
		int i=0;
		while (i<interactionList.size() && found==-1){
			if (interactionList.get(i).size()==seq.length){
				if (interactionList.get(i).isEqual(seq)) found=i;
			}
			i++;
		}
		
		if (found==-1){
			interactionList.add(new Composite(seq));
			additional.add(0f);
			found=interactionList.size()-1;
		}

		return found;
	}
	
	//////////////////////////////////
	// generate the predicted memories
	private void prediction(){
		
		// reset prediction maps
		for (int m=0;m<agent.interactionList.nbInteraction();m++){
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int n=0;n<9;n++){
						for (int k=0;k<3;k++){
							predictionMap.get(m)[i][j][n][k]=0f;
						}
					}
				}
			}	
		}
		
		// generate prediction maps
		for (int inter=0;inter<agent.interactionList.nbInteraction();inter++){
			for (int i=0;i<InteractionList.size1;i++){
				for (int j=0;j<InteractionList.size2*2;j++){
					int x=(int)(agent.interactionList.getInteraction(inter).vectorFlow[i][j][0]);
					int y=(int)(agent.interactionList.getInteraction(inter).vectorFlow[i][j][1]);

					// predicted map for each movement
					
					// forward-left
					if (x+(int)(motion*Math.sqrt(2))>=0 && x+(int)(motion*Math.sqrt(2))<InteractionList.size1 
					 && y-(int)(motion*Math.sqrt(2))>=0 && y-(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x+(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][0][k])>Math.abs(predictionMap.get(inter)[i][j][0][k])){
								predictionMap.get(inter)[i][j][0][k]=spaceMap[x+(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][0][k];
							}
						}
					}
					
					// left
					if (x+motion>=0 && x+motion<InteractionList.size1 && y>=0 && y<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x+(int)motion][y][1][k])>Math.abs(predictionMap.get(inter)[i][j][1][k])){
								predictionMap.get(inter)[i][j][1][k]=spaceMap[x+(int)motion][y][1][k];
							}
						}
					}
					
					// backward-left
					if (x+(int)(motion*Math.sqrt(2))>=0 && x+(int)(motion*Math.sqrt(2))<InteractionList.size1 
					 && y+(int)(motion*Math.sqrt(2))>=0 && y+(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x+(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][2][k])>Math.abs(predictionMap.get(inter)[i][j][2][k])){
								predictionMap.get(inter)[i][j][2][k]=spaceMap[x+(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][2][k];
							}
						}
					}
					
					// forward
					if (x>=0 && x<InteractionList.size1 && y-(int)motion>=0 && y-(int)motion<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x][y-(int)motion][3][k])>Math.abs(predictionMap.get(inter)[i][j][3][k])){
								predictionMap.get(inter)[i][j][3][k]=spaceMap[x][y-(int)motion][3][k];
							}
						}
					}
					
					// no movement
					if (x>=0 && x<InteractionList.size1 && y>=0 && y<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x][y][4][k])>Math.abs(predictionMap.get(inter)[i][j][4][k])){
								predictionMap.get(inter)[i][j][4][k]=spaceMap[x][y][4][k];
							}
						}
					}
					
					
					// backward
					if (x>=0 && x<InteractionList.size1 && y+(int)motion>=0 && y+(int)motion<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x][y+(int)motion][5][k])>Math.abs(predictionMap.get(inter)[i][j][5][k])){
								predictionMap.get(inter)[i][j][5][k]=spaceMap[x][y+(int)motion][5][k];
							}
						}
					}
					
					// forward-right
					if (x-(int)(motion*Math.sqrt(2))>=0 && x-(int)(motion*Math.sqrt(2))<InteractionList.size1 
					 && y-(int)(motion*Math.sqrt(2))>=0 && y-(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x-(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][6][k])>Math.abs(predictionMap.get(inter)[i][j][6][k])){
								predictionMap.get(inter)[i][j][6][k]=spaceMap[x-(int)(motion*Math.sqrt(2))][y-(int)(motion*Math.sqrt(2))][6][k];
							}
						}
					}
					
					// right
					if (x-motion>=0 && x+(int)motion<InteractionList.size1 && y>=0 && y<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x-(int)motion][y][7][k])>Math.abs(predictionMap.get(inter)[i][j][7][k])){
								predictionMap.get(inter)[i][j][7][k]=spaceMap[x-(int)motion][y][7][k];
							}
						}
					}
					
					// backward-right
					if (x-(int)(motion*Math.sqrt(2))>=0 && x-(int)(motion*Math.sqrt(2))<InteractionList.size1 
					 && y+(int)(motion*Math.sqrt(2))>=0 && y+(int)(motion*Math.sqrt(2))<InteractionList.size2*2){
						for (int k=0;k<3;k++){
							if (Math.abs(spaceMap[x-(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][8][k])>Math.abs(predictionMap.get(inter)[i][j][8][k])){
								predictionMap.get(inter)[i][j][8][k]=spaceMap[x-(int)(motion*Math.sqrt(2))][y+(int)(motion*Math.sqrt(2))][8][k];
							}
						}
					}
				}
			}
			
			// permutation of movement memories when the agent rotates
			float[][][] tempMap=new float[InteractionList.size1][InteractionList.size2*2][3];
			
			// turn left 90°
			if (inter==4){
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							tempMap[i][j][c]=predictionMap.get(inter)[i][j][3][c];			// save forward map
						}
					}
				}
				permuteMaps(1,3,inter);
				permuteMaps(5,1,inter);
				permuteMaps(7,5,inter);
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							predictionMap.get(inter)[i][j][7][c]=tempMap[i][j][c];			// save forward map
						}
					}
				}
				
				
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							tempMap[i][j][c]=predictionMap.get(inter)[i][j][0][c];			// save forward map
						}
					}
				}
				permuteMaps(2,0,inter);
				permuteMaps(8,2,inter);
				permuteMaps(6,8,inter);
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							predictionMap.get(inter)[i][j][6][c]=tempMap[i][j][c];			// save forward map
						}
					}
				}
			}
			
			
			// turn right 90°
			if (inter==3){
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							tempMap[i][j][c]=predictionMap.get(inter)[i][j][3][c];			// save forward map
						}
					}
				}
				permuteMaps(7,3,inter);
				permuteMaps(5,7,inter);
				permuteMaps(1,5,inter);
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							predictionMap.get(inter)[i][j][1][c]=tempMap[i][j][c];			// save forward map
						}
					}
				}
				
				
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							tempMap[i][j][c]=predictionMap.get(inter)[i][j][0][c];			// save forward map
						}
					}
				}
				permuteMaps(6,0,inter);
				permuteMaps(8,6,inter);
				permuteMaps(2,8,inter);
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							predictionMap.get(inter)[i][j][2][c]=tempMap[i][j][c];			// save forward map
						}
					}
				}
			}
			
			
			// turn left 45°
			if (inter==6){
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							tempMap[i][j][c]=predictionMap.get(inter)[i][j][3][c];			// save forward map
						}
					}
				}
				permuteMaps(0,3,inter);
				permuteMaps(1,0,inter);
				permuteMaps(2,1,inter);
				permuteMaps(5,2,inter);
				permuteMaps(8,5,inter);
				permuteMaps(7,8,inter);
				permuteMaps(6,7,inter);
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							predictionMap.get(inter)[i][j][6][c]=tempMap[i][j][c];			// save forward map
						}
					}
				}
			}
			
			// turn right 45°
			if (inter==5){
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							tempMap[i][j][c]=predictionMap.get(inter)[i][j][3][c];			// save forward map
						}
					}
				}
				permuteMaps(6,3,inter);
				permuteMaps(7,6,inter);
				permuteMaps(8,7,inter);
				permuteMaps(5,8,inter);
				permuteMaps(2,5,inter);
				permuteMaps(1,2,inter);
				permuteMaps(0,1,inter);
				for (int i=0;i<InteractionList.size1;i++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int c=0;c<3;c++){
							predictionMap.get(inter)[i][j][0][c]=tempMap[i][j][c];			// save forward map
						}
					}
				}
			}
			
		}
	}
	
	
	////////////////////////
	// send m1 map to m2 map
	public void permuteMaps(int m1, int m2){
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2*2;j++){
				for (int c=0;c<3;c++){
					spaceMap[i][j][m2][c]=spaceMap[i][j][m1][c];			// save forward map
				}
			}
		}
	}
	
	public void permuteMaps(int m1, int m2, int inter){
		for (int i=0;i<InteractionList.size1;i++){
			for (int j=0;j<InteractionList.size2*2;j++){
				for (int c=0;c<3;c++){
					predictionMap.get(inter)[i][j][m2][c]=predictionMap.get(inter)[i][j][m1][c];			// save forward map
				}
			}
		}
	}
	
	
	public int nbInteraction(){
		return interactionList.size();
	}
	
	public Composite getInteraction(int i){
		return interactionList.get(i);
	}
	
	public float getPredictionMap(int m, int i, int j, int n, int c){
		return predictionMap.get(m)[i][j][n][c];
	}
	
	public float getAdditional(int i){
		return additional.get(i);
	}
	
	public PrimitiveInteraction getTimeline(int i){
		return timeline[i];
	}
	
	/////////////////////////////////////////////////////////////////////
	public void save() {
		System.out.println("=====================prepare to save...======================");
		
		// save second level patterns
		
		String fileName = Observer.path+"spaceMemory/spaceMemoryV9_2 ("+timeSize+") "+agent.getIdent()+" "+Prey.speed+".txt";
		
		try {
			PrintWriter file  = new PrintWriter(new FileWriter(fileName));
			
			// save counter
			file.println(agent.action.getNbStep());
			file.println("***");
			
			// save patterns
			for (int i=0;i<interactionList.size();i++){
				
				// save the sequence
				for (int j=0;j<interactionList.get(i).size();j++){
					for (int k=0;k<agent.interactionList.nbInteraction();k++){
						if (interactionList.get(i).sequence[j].isEqual(agent.interactionList.getInteraction(k))) file.print(k+" ");
					}
					file.println();
					
					// save condition pattern
					for (int m=0;m<10;m++){
						for (int k=0;k<InteractionList.length;k++){
							file.print(interactionList.get(i).pattern.pattern[m][k]+" ");
						}
					}
					file.println();
				}
			}
			file.close();
			System.out.println("memory file saved");
		}
		catch (Exception e) {e.printStackTrace();}
	}
	
	
	private void load(){

		PrimitiveInteraction[] temp=new PrimitiveInteraction[1];
		for (int i=0;i<agent.interactionList.nbInteraction();i++){
			temp[0]=agent.interactionList.getInteraction(i);
			interactionList.add(new Composite(temp));
			
			predictionMap.add(new Float[InteractionList.size1][InteractionList.size2*2][9][3]);
			for (int l=0;l<InteractionList.size1;l++){
				for (int j=0;j<InteractionList.size2*2;j++){
					for (int m=0;m<9;m++){
						for (int k=0;k<3;k++){
							predictionMap.get(predictionMap.size()-1)[l][j][m][k]=0f;
						}
					}
				}
			}
			additional.add(0f);
		}
		
	}
	
	public void loadFile(){
		String fileName = Observer.path+"spaceMemory/spaceMemoryV9_2 ("+timeSize+") "+agent.getIdent()+" "+Prey.speed+".txt";
		String[] elements;

		try {
			InputStream ips=new FileInputStream(fileName); 
			InputStreamReader ipsr=new InputStreamReader(ips);
			BufferedReader br=new BufferedReader(ipsr);
			String line;
			
			// set agent's step counter
			line=br.readLine();
			agent.action.setNbstep(Integer.parseInt(line));

			// separator
			line=br.readLine();

			// load patterns
			while ((line=br.readLine())!=null){
				elements=line.split(" ");
				
				// build sequences
				PrimitiveInteraction[] temp=new PrimitiveInteraction[elements.length];
				for (int i=0;i<elements.length;i++){
					temp[i]=agent.interactionList.getInteraction(Integer.parseInt(elements[i]));
				}
				learn(temp);
				
				line=br.readLine();
				elements=line.split(" ");
				
				// get condition pattern
				float max=0;
				for (int m=0;m<10;m++){
					for (int i=0;i<InteractionList.length;i++){
						interactionList.get(interactionList.size()-1).pattern.pattern[m][i]=Float.parseFloat(elements[i+m*InteractionList.length]);
						if (Math.abs(Float.parseFloat(elements[i]))>max) max=Math.abs(Float.parseFloat(elements[i]));
					}
				}
				interactionList.get(interactionList.size()-1).pattern.maxPattern=max;
				interactionList.get(interactionList.size()-1).pattern.maxPattern1=max;
				interactionList.get(interactionList.size()-1).pattern.maxPattern2=max;
				
				predictionMap.add(new Float[InteractionList.size1][InteractionList.size2*2][9][3]);
				for (int l=0;l<InteractionList.size1;l++){
					for (int j=0;j<InteractionList.size2*2;j++){
						for (int m=0;m<9;m++){
							for (int k=0;k<3;k++){
								predictionMap.get(predictionMap.size()-1)[l][j][m][k]=0f;
							}
						}
					}
				}
				additional.add(0f);
			}
		} 
		catch (Exception e) {
			System.out.println("no file found");
			load();
		}
	}
}
