package version9_2_2_1.agnosticMemory;

import version9_2_2_1.Interface.InteractionList;
import version9_2_2_1.Interface.PrimitiveInteraction;
import version9_2_2_1.platform.Agent;

/**
 * Environmental context.
 * This class can also generate a predicted and an improved context
 * @author simon gay
 */
public class EnvironmentContext{
	
	private Agent agent;
	
	public float[][] enactedEnsembles;				// enacted ensembles E_t and E_{t-1}
	
	public float[] predictedContext;
	
	public short[][] counter;
	
	public EnvironmentContext(Agent a){
		
		agent = a;
		
		enactedEnsembles=new float[2][InteractionList.length];
		predictedContext=new float[InteractionList.length];
		counter=new short[7][InteractionList.length];
		for (short i=0;i<InteractionList.length;i++){
			enactedEnsembles[0][i]=0;
			enactedEnsembles[1][i]=0;
			predictedContext[i]=0;
			
			for (byte j=0;j<7;j++){
				counter[j][i]=25;
			}
		}
	}
	
	
	/**
	 * update the environmental context
	 * @param inter : last enacted primitive interaction
	 */
	public void updateContext(PrimitiveInteraction inter, float[] enacted){
		
		// update timeline
		for (short i=0;i<InteractionList.length;i++){
			enactedEnsembles[1][i]=enactedEnsembles[0][i];
			enactedEnsembles[0][i]=enacted[i];
		}
		
		// update signatures
		agent.signatureList.learn(enactedEnsembles[1],enacted , inter);
		
		// compute predictions
		agent.signatureList.setPredictions(enactedEnsembles[0]);
		
		// set counters
		byte id=0;
		if (inter.getAction().getName().equals(">")){
			if (inter.getPerception().getId()==1) id=1;
			else if (inter.getPerception().getId()==2) id=2;
			else id=0;
		}
		else if (inter.getAction().getName().equals("v")) id=3;
		else if (inter.getAction().getName().equals("^")) id=4;
		else if (inter.getAction().getName().equals("\\"))id=5;
		else if (inter.getAction().getName().equals("/")) id=6;
		
		for (short i=0;i<InteractionList.length;i++){
			if (enactedEnsembles[1][i]==1 && counter[id][i]>1) counter[id][i]--;
		}
		
		// set predicted context
		for (short i=0;i<InteractionList.length-1;i++){
			predictedContext[i]=agent.signatureList.get(i).getPrediction();
		}
		
	}
	
}
