package version9_2_2_1.agnosticMemory;

import version9_2_2_1.Interface.InteractionList;
import version9_2_2_1.Interface.PrimitiveInteraction;

/**
 * signature of an interaction
 * @author simon gay
 */
public class Signature {

	private PrimitiveInteraction interaction;	// primary interaction or associated interaction used to define selected action 
	
	public short[] signature;					// signature of the interaction
	
	public short maxPattern1;					// maximum weight (used to normalize values in displayer)
	public short maxPattern2;
	
	public short accurate=0;
	public short nbTest=0;
	
	private float prediction=0;
	
	public float average_delta1=0;
	public float average_delta2=0;
	
	public Signature(PrimitiveInteraction inter){
		interaction=inter;
		
		signature=new short[InteractionList.length];
		for (short i=0;i<InteractionList.length;i++){
			signature[i]=0;
		}
		maxPattern1=0;
		maxPattern2=0;

	}
	
	/**
	 * predict the result of an interaction in a given context
	 * @param img : environmental context
	 * @param write : if true, store the prediction
	 * @return the certitude of success (1->absolute certitude of success, -1-> absolute certitude of failure
	 */
	public float prediction(float[] img, boolean write){
		float result=0;
		for (short i=0;i<InteractionList.length;i++){
			result+=signature[i]*img[i];
		}
		result=result/1000;
		result= (float) ( 1 / (1+exp(-result)))*2-1;
		
		if (write) prediction=result;
		
		return result;
	}
	
	/**
	 * integrate the result of enaction cycle
	 * @param img : environmental context E_{t-1} 
	 * @param output result of enaction (success->1, failure->-1)
	 */
	public void learn(float[] img, float output){
		maxPattern1=0;
		maxPattern2=0;
		float delta=output-prediction;

			
		for (short i=0;i<InteractionList.length;i++){
			if (signature[i]>-32000 && signature[i]<32000) 
				signature[i]+= 0.05 * delta * img[i] * 1000;
			else System.out.println("weight out of limits");
			if (i< InteractionList.length-8 && Math.abs(signature[i])>maxPattern1) maxPattern1=(short) Math.abs(signature[i]);
			if (i>=InteractionList.length-8 && Math.abs(signature[i])>maxPattern2) maxPattern2=(short) Math.abs(signature[i]);
		}
		
		// limit the bias value, as the range of weights is limited by the 'short' type
		if (signature[InteractionList.length-1]> 6000){
			signature[InteractionList.length-1]= 6000;
			for (int i=0;i<InteractionList.length-1;i++){
				if (img[i]>0) signature[i]=(short)((float)signature[i]*0.9f);
			}
		}
		if (signature[InteractionList.length-1]<-6000){
			signature[InteractionList.length-1]=-6000;
			for (int i=0;i<InteractionList.length-1;i++){
				if (img[i]>0) signature[i]=(short)((float)signature[i]*0.9f);
			}
		}
		
		// compute average delta to detect stabilization
		if (delta>=0) average_delta1= (100*average_delta1 + Math.abs(delta))/101;
		if (delta<=0) average_delta2= (100*average_delta2 + Math.abs(delta))/101;
		
		if (prediction*output>0.1){
			if (accurate<5) accurate++;
		}
		else accurate=0;
		
		if (output>0) nbTest++;
	}
	
	public boolean isAccurate(){
		return accurate>=5 || Math.abs(prediction)>0.2;
	}
	
	public float getPrediction(){
		return prediction;
	}
	
	public PrimitiveInteraction interaction(){
		return interaction;
	}
	
	// fast exp function approximation
	public static double exp(double val) {
	    final long tmp = (long) (1512775 * val + (1072693248 - 60801));
	    return Double.longBitsToDouble(tmp << 32);
	}
	
}
