package version9_2_2_1.display;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import version9_2_2_1.platform.Agent;
import version9_2_2_1.Interface.InteractionList;
import version9_2_2_1.agnosticMemory.Signature;

/**
 * Frame to display images of signatures
 * @author simon
 */

/* - inherited from EnvPanel :
 *   Agent agent      : pointer to the agent
 */
public class MovementPanel extends EnvPanel implements MouseListener{

	private static final long serialVersionUID = 1L;
	
	private float[][][][][] maps;						// sequence of signature images
	private short[] selection;
	private short imageStep;
	private byte color;
	private byte nbStep=5;
	
	private short clic_x=0;
	private short clic_y=0;
	
	private int size1=InteractionList.size1;
	private int size2=InteractionList.size2;
	
	public MovementPanel(Agent a){
		super(a);
		addMouseListener(this);
		
		maps=new float[nbStep][11][6][size1][size2];
		
		selection=new short[7];
		for (int i=0;i<nbStep;i++){
			selection[i]=0;
		}
		
		imageStep=0;
		color=0;
		
		setSignature();
		fillMap();
	}
	
	public void setAgent(Agent a){
		agent=a;
	}
	
	public void paintComponent(Graphics g){
		g.setColor(Color.white);
		
		g.fillRect(0, 0, 1160, 705);
		
		
		// draw color selector
		for (byte j=0;j<3;j++){
			if (color==j) g.setColor(Color.red);
			else g.setColor(Color.black);
			
			g.fillRect(40, 140 + 15*j, 10, 10);
		}
		
		for (short i=0;i<nbStep;i++){
			
			// draw selector
			for (byte j=0;j<7;j++){
				if (i==0 || j<6){
					if (selection[i]==j){
						if (i==imageStep) g.setColor(Color.green);
						else g.setColor(Color.red);
					}
					else g.setColor(Color.black);
					
					g.fillRect(40+40*i, 10 + 15*j, 10, 10);
				}
			}
		}
			
		// draw maps
		for (byte l=0;l<9;l++){
			for (byte n=0;n<6;n++){
				for (short j=0;j<size1;j++){
					for (short k=0;k<size2;k++){

						if (l==4){
							g.setColor(new Color(Math.min(1, Math.max(0, (maps[imageStep][10][n][j][k]/2+0.5f))),
									 			 Math.min(1, Math.max(0, (maps[imageStep][ 9][n][j][k]/2+0.5f))),
									 			 Math.min(1, Math.max(0, (maps[imageStep][ 4][n][j][k]/2+0.5f)))));
						}
						else{
							g.setColor(new Color(0.5f, 0.5f, Math.min(1, Math.max(0, (maps[imageStep][ l][n][j][k]/2+0.5f)))));
						}
						g.fillRect(10+125*l+4*j, 200+(size2*4- 4*k) +70*n,4,4);
					}
				}
			}
		}
	}
	
	public void setSignature(){
		// fill initial signature
		float max=0;
		if (selection[0]>=0){
			for (byte l=0;l<11;l++){
				for (byte n=0;n<6;n++){
					for (short i=0;i<size1;i++){
						for (short j=0;j<size2;j++){
							if (Math.abs(agent.signatureList.get(InteractionList.nbDF+selection[0]).signature[i*size2+j+n*size1*size2 +l*InteractionList.nbType ])>200
							 //&& Math.abs(agent.signatureList.get(InteractionList.nbDF+selection[0]).signature[i*size2+j+n*size1*size2 +l*InteractionList.nbType ])>(float)(agent.signatureList.get(InteractionList.nbDF+selection[0]).maxPattern1)*0.2f){
							){
								maps[0][l][n][i][j]=(float)(agent.signatureList.get(InteractionList.nbDF+selection[0]).signature[i*size2+j+n*size1*size2 +l*InteractionList.nbType ])/1000;
								if (maps[0][l][n][i][j]>max) max=maps[0][l][n][i][j];
							}
							else{
								maps[0][l][n][i][j]=0;
							}
						}
					}
				}
			}
		}
		else{
			for (byte l=0;l<11;l++){
				for (byte n=0;n<6;n++){
					for (short i=0;i<size1;i++){
						for (short j=0;j<size2;j++){
							
							if (Math.abs(agent.signatureList.get(-selection[0]-1).signature[i*size2+j+n*size1*size2 +l*InteractionList.nbType ])>200
							// && Math.abs(agent.signatureList.get(-selection[0]-1).signature[i*size2+j+n*size1*size2 +l*InteractionList.nbType ])>(float)(agent.signatureList.get(-selection[0]-1).maxPattern1)*0.2f){
							){
								maps[0][l][n][i][j]=(float)(agent.signatureList.get(-selection[0]-1).signature[i*size2+j+n*size1*size2 +l*InteractionList.nbType ])/1000;
								if (maps[0][l][n][i][j]>max) max=maps[0][l][n][i][j];
							}
							else{
								maps[0][l][n][i][j]=0;
							}
						}
					}
				}
			}
		}
		
		
		if (max>0.1){
			for (byte l=0;l<11;l++){
				for (byte n=0;n<6;n++){
					for (short i=0;i<size1;i++){
						for (short j=0;j<size2;j++){
								maps[0][l][n][i][j]=maps[0][l][n][i][j]/max;
						}
					}
				}
			}
		}
	}
	
	
	public void fillMap(){
		// generate moved signatures
		int a=1;
		while (a<=imageStep){
		
			// reset map
			float max=0;
			for(byte l=0;l<11;l++){
				for (byte n=0;n<6;n++){
					for (short i=0;i<size1;i++){
						for (short j=0;j<size2;j++){
							maps[a][l][n][i][j]=0;
						}
					}
				}
			}
			
			// compute map
			for(byte l=0;l<11;l++){
				for (short n=selection[a];n<selection[a]+1;n++){
					for (short i=0;i<size1;i++){
						for (short j=0;j<size2;j++){
							
							// get the signature
							Signature sig=agent.signatureList.get(i*size2+j+n*size2*size1+ l*InteractionList.nbType );
								
							//if (sig.average_delta1<0.7 && sig.average_delta2<0.7){
								
								// apply the signature
								for (byte l2=0;l2<11;l2++){
									for (byte n2=0;n2<6;n2++){
										for (short i2=0;i2<size1;i2++){
											for (short j2=0;j2<size2;j2++){
												if (Math.abs(sig.signature[i2*size2+j2+n2*size2*size1+l2*InteractionList.nbType])>200 
												//if (Math.abs(maps[a-1][l][n][i][j]*sig.signature[i2*size2+j2+n2*size2*size1+l2*InteractionList.nbType])>200 
												//if (Math.abs(maps[a-1][l][n][i][j]*sig.signature[i2*size2+j2+n2*size2*size1+l2*InteractionList.nbType])>Math.abs(maps[a][l2][n2][i2][j2])
												){
													maps[a][l2][n2][i2][j2]+= maps[a-1][l][n][i][j] * sig.signature[i2*size2+j2+n2*size2*size1+l2*InteractionList.nbType];
													if (Math.abs(maps[a][l2][n2][i2][j2])>max){
														max = Math.abs(maps[a][l2][n2][i2][j2]);
													}
												}
											}
										}
									}
								}
							//}
						}
					}
				}
			}
			
			// normalise map
			if (max>1){
				for(byte l=0;l<11;l++){
					for (byte n=0;n<6;n++){
						for (short i=0;i<size1;i++){
							for (short j=0;j<size2;j++){
								maps[a][l][n][i][j]=2*maps[a][l][n][i][j]/max;
							}
						}
					}
				}
			}
			a++;
		}
	}
	
	
	///////////////////////////////////////////////////////////////////////////
	public void mouseClicked(MouseEvent e) {

		clic_x=(short) e.getX();
		clic_y=(short) e.getY();

		boolean found=false;
		
		for (byte i=0;i<nbStep;i++){
			for (byte j=0;j<7;j++){
				if (i==0 || j<6){
					if (clic_x>=38+40*i && clic_x<52+40*i && clic_y>=8+15*j && clic_y<22+15*j){
						selection[i]=j;
						imageStep=i;
						found=true;
					}
				}
			}
		}

		for (byte j=0;j<3;j++){
			if (clic_x>=38 && clic_x<52 && clic_y>=138+15*j && clic_y<152+15*j){
				color=j;
				found=true;
			}
		}
		
		// visual interaction selection
		if (!found){
			for (byte l=0;l<9;l++){
				for (byte n=0;n<6;n++){
					for (short j=0;j<size1;j++){
						for (short k=0;k<size2;k++){
							
							if (clic_x>10+125*l+4*j && clic_x<14+125*l+4*j && clic_y>200+(size2*4-4*k)+70*n && clic_y<204+(size2*4-4*k)+70*n){
								selection[0]=(short)( -1-(j*size2+k+n*size1*size2 +l*InteractionList.nbType));
								found=true;
							}
						}
					}
				}
			}
		}

		
		if (found) setSignature();
		fillMap();
	}

	@Override
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e){}
}
