package version9_2.spaceMemory;

import version9_2.Interface.InteractionList;
import version9_2.Interface.PrimitiveInteraction;
import version9_2.platform.Agent;

/**
 * Decision system.
 * This class selects the intended interactions
 * @author simon
 */
public class Decision {

	private Agent agent;
	
	private Composite lastSequence=null;
	private PrimitiveInteraction intention=null;

	private int sequenceIndex=-1;
	private boolean learning=true;  // enable learning
	private int decisionType=0;
	
	public static float incertitude=0.2f;
	
	private Thread[] t_utility;
	private float[] utilities;
	
	///////////////////////////////////////////////////////////
	public Decision(Agent a){
		agent=a;
		
		t_utility=new Thread[7];
		utilities=new float[7];
	}
	
	/**
	 * decision of the next interaction
	 * @return the intended interaction
	 */
	public PrimitiveInteraction decision(){
		System.out.println();System.out.println();

		sequenceIndex--;
		
		// if sequence completed, select another one
		if (sequenceIndex<0){

			float min=0;

			// get the interaction with the lower certitude (in absolute value)
			for (int i=0;i<agent.spaceMemory.nbInteraction();i++){
				if (isPathPossible(agent.spaceMemory.getInteraction(i))){
					if (Math.abs(1-Math.abs(agent.spaceMemory.getInteraction(i).getPrediction()))>min){
						min=Math.abs(1-Math.abs(agent.spaceMemory.getInteraction(i).getPrediction()));
					}
				}
			}
			
			// select an interaction for exploitation
			if (min<incertitude || !learning){
				int imax=-1;
				float max=-1000;
				for (int i=0;i<agent.spaceMemory.nbInteraction();i++){
					if (isPathPossible(agent.spaceMemory.getInteraction(i))){
						
						// get the interaction with the greatest valence
						if (agent.spaceMemory.getInteraction(i).getPrediction()>=0
						 &&(agent.spaceMemory.getInteraction(i).getValence()+agent.spaceMemory.getAdditional(i) *SpaceMemory.memoryCoef>max)){
							imax=i;
							max=agent.spaceMemory.getInteraction(i).getValence()+agent.spaceMemory.getAdditional(i) *SpaceMemory.memoryCoef;
						}
					}
				}
				
				lastSequence=agent.spaceMemory.getInteraction(imax);
				decisionType=1;
			}
			// select an interaction for learning
			else{
				int imax=-1;
				float max=-1000;
				for (short i=0;i<7;i++) t_utility[i]=new UtilityThread(i);
				for (short i=0;i<7;i++) t_utility[i].start();
				try {
					for (short i=0;i<7;i++) t_utility[i].join();
				} catch (InterruptedException e){e.printStackTrace();}
				
				for (short i=0;i<7;i++){
					System.out.println(" +++ "+i+ " ; "+utilities[i] + " ; "+agent.spaceMemory.getInteraction(i).getPrediction());
					if (utilities[i]>max && utilities[i]!=-1){
						max=utilities[i];
						imax=i;
					}
				}
				
				lastSequence=agent.spaceMemory.getInteraction(imax);
				decisionType=2;
			}
			sequenceIndex=lastSequence.size()-1;
			System.out.println("+++++ select sequence "+lastSequence.getName(0));
		}

		intention=lastSequence.sequence[sequenceIndex];
		
		System.out.println("select interaction "+intention.getName()+" element of "+lastSequence.getName(0));
		
		return intention;
	}
	
	
	public void check(PrimitiveInteraction enacted){
		if (!intention.isEqual(enacted)){
			sequenceIndex=-1;
		}
	}
	
	private boolean isPathPossible(Composite inter){
		if (inter.size()==1) return true;
		else{
			boolean ret=true;
			for (int i=0;i<agent.spaceMemory.nbInteraction();i++){
				if (inter.isPath(agent.spaceMemory.getInteraction(i))){
					if (agent.spaceMemory.getInteraction(i).getPrediction()<0.8) ret=false;
				}
			}
			return ret;
		}
	}
	
	public int getDecisionType(){
		return decisionType;
	}
	
	public boolean learning(){
		return learning;
	}
	
	public void setLearning(){
		learning= !learning;
	}
	
	
	private void getUtility(int id){
		float sum=0;
		
		if (agent.spaceMemory.getInteraction(id).getPrediction()>-0.5){
			
			for (int i=0;i<9;i++){
				for (int j=0;j<InteractionList.length-1;j++){
					if (agent.spaceMemory.enactedEnsemblesM[0][i][j]>0){
						sum+= agent.spaceMemory.counters[id][i][j];
					}
				}
			}
			utilities[id]=sum + (1-Math.abs(agent.spaceMemory.getInteraction(id).getPrediction()));
		}
		else utilities[id]=-1;
	}
	
	

	/**
	 * Get the last intended interaction
	 * @return the last selected intended interaction
	 */
	public PrimitiveInteraction getIntention(){
		return intention;
	}
	
	
	//////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////
	
	private class UtilityThread extends Thread {
		
		private short id;
		
		public UtilityThread(short i){
			super();
			id=i;
		}
		public void run(){
			getUtility(id);
		}
	}
	
}
